# -*- coding: utf-8 -*-
try:
    from django.urls import reverse_lazy
except ImportError:
    from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect, reverse
from django.views.generic import CreateView, DeleteView, DetailView, FormView, ListView, UpdateView

from .forms import PluginCreateForm
from .models import AddressPlugin, ImagePlugin, Menu, Page, Plugin, TextPlugin
from . import plugin_map


class MenuCreateView(CreateView):
    model = Menu
    fields = ['title', 'page', 'children', 'root', 'enabled']
    success_url = reverse_lazy('page_list')


class MenuChildCreateView(CreateView):
    model = Menu
    fields = ['title', 'page', 'children', 'root', 'enabled']

    def form_valid(self, form):
        parent = Menu.objects.get(pk=int(self.kwargs['pk']))
        form.instance.save()
        parent.children.add(form.instance)
        return redirect('menu_detail', pk=form.instance.pk)


class MenuDetailView(DetailView):
    model = Menu


class MenuDeleteView(DeleteView):
    model = Menu
    success_url = reverse_lazy('page_list')

    def delete(self, request, *args, **kwargs):
        menu = Menu.objects.get(pk=kwargs['pk'])
        for menu in menu.children.all():
            menu.delete()
        return super(MenuDeleteView, self).delete(self, request, args, kwargs)


class MenuUpdateView(UpdateView):
    fields = ['title', 'page', 'children', 'root', 'enabled']
    model = Menu

    def get_success_url(self):
        return reverse('menu_detail', kwargs={'pk': self.object.pk})


class PageListView(ListView):
    model = Page

    def get_context_data(self, **kwargs):
        context = super(PageListView, self).get_context_data(**kwargs)
        context['menus'] = Menu.objects.filter(root=True)
        return context


class PageCreateView(CreateView):
    model = Page
    fields = ['name', ]
    success_url = reverse_lazy('page_list')


class PageUpdateView(UpdateView):
    template_name = 'cow/page_update.html'
    model = Page
    fields = ['name', ]


class PageDeleteView(DeleteView):
    model = Page
    success_url = reverse_lazy('page_list')

    def delete(self, request, *args, **kwargs):
        page = Page.objects.get(pk=kwargs['pk'])
        for page_plugin in page.plugins.all():
            page_plugin.content_object.delete()
            page_plugin.delete()
        return super(PageDeleteView, self).delete(self, request, args, kwargs)


class PluginCreateView(FormView):
    template_name = 'cow/plugin_create.html'
    form_class = PluginCreateForm

    def form_valid(self, form):
        plugin_class = plugin_map[form.cleaned_data['plugin_type']]
        plugin_instance = plugin_class.objects.create()
        plugin = Plugin.objects.create(content_object=plugin_instance)
        page_id = int(filter(str.isdigit, str(self.request.path)))
        page = Page.objects.get(pk=page_id)
        page.plugins.add(plugin)
        return redirect(plugin_instance.get_absolute_url())


class PluginDeleteView(DeleteView):
    model = Plugin
    success_url = reverse_lazy('page_list')

    def delete(self, request, *args, **kwargs):
        plugin = Plugin.objects.get(pk=kwargs['pk'])
        plugin.content_object.delete()
        return super(PluginDeleteView, self).delete(self, request, args, kwargs)

    def get_success_url(self):
        queryset = self.object.page_set.all()
        if queryset:
            page = queryset.first()
            return reverse('page_update', args=[page.id])
        return reverse('page_list')


class AddressPluginUpdateView(UpdateView):
    fields = ['address', ]
    model = AddressPlugin

    def get_success_url(self):
        return reverse('address_plugin_edit', kwargs={'pk': self.object.pk})


class ImagePluginUpdateView(UpdateView):
    fields = ['image', ]
    model = ImagePlugin

    def get_success_url(self):
        return reverse('image_plugin_edit', kwargs={'pk': self.object.pk})


class TextPluginUpdateView(UpdateView):
    fields = ['content', ]
    model = TextPlugin

    def get_success_url(self):
        return reverse('text_plugin_edit', kwargs={'pk': self.object.pk})



# -*- coding: utf-8 -*-
from django.conf.urls import include, url
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import login_required
from rest_framework import routers
from . import views
from . import api


router = routers.DefaultRouter()
router.register(r'pages', api.PageViewSet)
router.register(r'menus', api.MenuViewSet)


urlpatterns = [
    url('^accounts/login/$', auth_views.login, {'template_name': 'admin/login.html'}, name='cow_login'),
    url('^api/', include(router.urls)),
    url('^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
    url(
        regex="^menu/create/$",
        view=login_required(views.MenuCreateView.as_view()),
        name='menu_create',
    ),
    url(
        regex="^menu/(?P<pk>\d+)/create/$",
        view=login_required(views.MenuChildCreateView.as_view()),
        name='menu_child_create',
    ),
    url(
        regex="^menu/delete/(?P<pk>\d+)/$",
        view=login_required(views.MenuDeleteView.as_view()),
        name='menu_delete',
    ),
    url(
        regex="^menu/update/(?P<pk>\d+)/$",
        view=login_required(views.MenuUpdateView.as_view()),
        name='menu_update',
    ),
    url(
        regex="^menu/(?P<pk>\d+)/$",
        view=login_required(views.MenuDetailView.as_view()),
        name='menu_detail',
    ),
    url(
        regex="^$",
        view=login_required(views.PageListView.as_view()),
        name='page_list',
    ),
    url(
        regex="^page/create/$",
        view=login_required(views.PageCreateView.as_view()),
        name='page_create',
    ),
    url(
        regex="^page/delete/(?P<pk>\d+)/$",
        view=login_required(views.PageDeleteView.as_view()),
        name='page_delete',
    ),
    url(
        regex="^page/(?P<pk>\d+)/$",
        view=login_required(views.PageUpdateView.as_view()),
        name='page_update',
    ),
    url(
        regex="^page/(?P<pk>\d+)/plugin/create/$",
        view=login_required(views.PluginCreateView.as_view()),
        name='plugin_create',
    ),
    url(
        regex="^plugin/delete/(?P<pk>\d+)/$",
        view=login_required(views.PluginDeleteView.as_view()),
        name='plugin_delete',
    ),
    url(
        regex="^plugin/text/(?P<pk>\d+)/$",
        view=login_required(views.TextPluginUpdateView.as_view()),
        name='text_plugin_edit',
    ),
    url(
        regex="^plugin/address/(?P<pk>\d+)/$",
        view=login_required(views.AddressPluginUpdateView.as_view()),
        name='address_plugin_edit',
    ),
    url(
        regex="^plugin/image/(?P<pk>\d+)/$",
        view=login_required(views.ImagePluginUpdateView.as_view()),
        name='image_plugin_edit',
    ),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '1.0.7'
readme = open('README.rst').read()

setup(
    name='cow',
    version=version,
    description="Yuge heifer-like bloated CMS - now with more django, S3, Lambda friendliness!",
    long_description=readme,
    author='Jon Robison',
    author_email='narfman0@gmail.com',
    url='https://github.com/narfman0/cow',
    packages=[
        'cow',
    ],
    include_package_data=True,
    install_requires=[
        'django',
        'djangorestframework',
        'pillow',
    ],
    license="MIT",
    zip_safe=False,
    keywords=['django', 'cow', 'cms'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
)



from django.conf import settings
from rest_framework import serializers, viewsets
from .models import Menu, Page, Plugin


# lolworthily high, also the max :)
MENU_DEPTH = getattr(settings, 'COW_API_MENU_DEPTH', 10)


class PageSerializer(serializers.ModelSerializer):
    class Meta:
        model = Page
        fields = ('id', 'name', 'plugins')

    def to_representation(self, instance):
        data = super(PageSerializer, self).to_representation(instance)
        plugins_api = []
        for plugin_id in data['plugins']:
            plugin = Plugin.objects.get(pk=int(plugin_id))
            plugins_api.append(plugin.content_object.api_serialize())
        data['plugins'] = plugins_api
        return data


class MenuSerializer(serializers.ModelSerializer):
    class Meta:
        depth = MENU_DEPTH
        model = Menu
        fields = ('id', 'title', 'enabled', 'root', 'children')


class PageViewSet(viewsets.ModelViewSet):
    queryset = Page.objects.all()
    serializer_class = PageSerializer


class MenuViewSet(viewsets.ModelViewSet):
    queryset = Menu.objects.filter(root=True)
    serializer_class = MenuSerializer
